/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export;

import edu.indiana.dlib.vfrbr.export.rdf.entityExporter.CorporateBodyRDFExporter;
import edu.indiana.dlib.vfrbr.export.rdf.entityExporter.ExpressionRDFExporter;
import edu.indiana.dlib.vfrbr.export.rdf.entityExporter.ManifestationRDFExporter;
import edu.indiana.dlib.vfrbr.export.rdf.entityExporter.PersonRDFExporter;
import edu.indiana.dlib.vfrbr.export.rdf.entityExporter.WorkRDFExporter;
import edu.indiana.dlib.vfrbr.persist.dao.CorporateBodyDAO;
import edu.indiana.dlib.vfrbr.persist.dao.DAOFactory;
import edu.indiana.dlib.vfrbr.persist.dao.ExpressionDAO;
import edu.indiana.dlib.vfrbr.persist.dao.ManifestationDAO;
import edu.indiana.dlib.vfrbr.persist.dao.PersonDAO;
import edu.indiana.dlib.vfrbr.persist.dao.WorkDAO;
import java.util.List;

public class RDFExport {

    private DAOFactory dao = new DAOFactory();

    public static void main(String[] args) {
        RDFExport rdfx = new RDFExport();
        rdfx.exportManifestations();
        rdfx.exportPeople();
        rdfx.exportCorporateBodies();
        rdfx.exportExpressions();
        rdfx.exportWorks();
    }

    public RDFExport() {
    }

    public void exportPeople() {
        PersonDAO personDAO = dao.newPersonDAO();
        List<Integer> personIds = personDAO.getAllIds();
        PersonRDFExporter pe = new PersonRDFExporter(personIds, "person", dao, personDAO);
        pe.handleExport();
    }

    public void exportWorks() {
        WorkDAO workDAO = dao.newWorkDAO();
        List<Integer> workIds = workDAO.getAllIds();
        WorkRDFExporter we = new WorkRDFExporter(workIds, "work", dao, workDAO);
        we.handleExport();
    }

    public void exportExpressions() {
        ExpressionDAO exprDAO = dao.newExpressionDAO();
        List<Integer> exprIds = exprDAO.getAllIds();
        ExpressionRDFExporter ee = new ExpressionRDFExporter(exprIds, "expression", dao, exprDAO);
        ee.handleExport();
    }

    public void exportManifestations() {
        ManifestationDAO manifDAO = dao.newManifestationDAO();
        List<Integer> manifIds = manifDAO.getAllIds();
        ManifestationRDFExporter me = new ManifestationRDFExporter(manifIds, "manifestation", dao, manifDAO);
        me.handleExport();
    }

    public void exportCorporateBodies() {
        CorporateBodyDAO corpDAO = dao.newCorporateBodyDAO();
        List<Integer> corpIds = corpDAO.getAllIds();
        CorporateBodyRDFExporter cbe = new CorporateBodyRDFExporter(corpIds, "corporatebody", dao, corpDAO);
        cbe.handleExport();
    }
}