/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export;

import edu.indiana.dlib.vfrbr.export.context.CorporateBodyContext;
import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.export.context.ExpressionContext;
import edu.indiana.dlib.vfrbr.export.context.ManifestationContext;
import edu.indiana.dlib.vfrbr.export.context.WorkContext;
import edu.indiana.dlib.vfrbr.export.context.StructureEmbodiedContext;
import edu.indiana.dlib.vfrbr.export.context.StructureRealizedContext;
import edu.indiana.dlib.vfrbr.export.context.PersonContext;
import edu.indiana.dlib.vfrbr.export.context.ResponsibleCreatedContext;
import edu.indiana.dlib.vfrbr.export.context.ResponsibleProducedContext;
import edu.indiana.dlib.vfrbr.export.context.ResponsibleRealizedContext;

import java.io.IOException;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class XMLExport {

    private static final Logger log = Logger.getLogger(XMLExport.class);

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
            throws IOException {

        XMLExport test = new XMLExport();

        if (log.isInfoEnabled()) {
            log.info(""
                    + "============"
                    + "Exporting started with ExportContext: \n"
                    + "  scheme:  " + XMLExportContext.getScheme() + "\n"
                    + "  level:   " + XMLExportContext.getLevel() + "\n"
                    + "  version: " + XMLExportContext.getVersion() + "\n"
                    + "============");
        }

        test.exportingTest();

    }

    public void exportingTest() {
        exportWorks();
        exportExpressions();
        exportManifestations();
        exportPersons();
        exportCorporateBodies();
        exportStructRealized();
        exportStructEmbodied();
        exportResponCreated();
        exportResponRealized();
        exportResponProduced();
    }

    public void exportWorks() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.WorkExporter(
                    new WorkContext()).exportWorks();
        } catch (Exception ex) {
        }
    }

    public void exportExpressions() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.ExpressionExporter(
                    new ExpressionContext()).exportExpressions();
        } catch (Exception ex) {
        }
    }

    public void exportManifestations() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.ManifestationExporter(
                    new ManifestationContext()).exportManifestations();
        } catch (Exception ex) {
        }
    }

    public void exportPersons() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.PersonExporter(
                    new PersonContext()).exportPersons();
        } catch (Exception ex) {
        }
    }

    public void exportCorporateBodies() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.CorporateBodyExporter(
                    new CorporateBodyContext()).exportCorporateBodies();
        } catch (Exception ex) {
        }
    }

    public void exportStructRealized() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.StructRealizedExporter(
                    new StructureRealizedContext()).exportRealizedThrough();
        } catch (Exception ex) {
        }
    }

    public void exportStructEmbodied() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.StructEmbodiedExporter(
                    new StructureEmbodiedContext()).exportEmbodiedIn();
        } catch (Exception ex) {
        }
    }

    public void exportResponCreated() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.ResponCreatedExporter(
                    new ResponsibleCreatedContext()).exportCreatedBy();
        } catch (Exception ex) {
        }
    }

    public void exportResponRealized() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.ResponRealizedExporter(
                    new ResponsibleRealizedContext()).exportRealizedBy();
        } catch (Exception ex) {
        }
    }

    public void exportResponProduced() {
        try {
            new edu.indiana.dlib.vfrbr.export.xml.exporter.ResponProducedExporter(
                    new ResponsibleProducedContext()).exportProducedBy();
        } catch (Exception ex) {
        }
    }
}
