/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.context;

import org.apache.log4j.Logger;

/**
 *  Context for a Work Export.
 *
 */
public class StructureEmbodiedContext {

    private static final Logger log =
            Logger.getLogger(StructureEmbodiedContext.class);

    /**
     * Stem for the export file name.
     */
    private String fileStem;

    /**
     * How many records to a file.
     */
    private int recsPerFile;

    /**
     * Sample to index, exclusive.
     */
    private boolean sampleExport;

    private int sampleFromIndex;

    private int sampleToIndex;

    /**
     * Create and initialize a new StructContext.
     */
    public StructureEmbodiedContext() {

        fileStem = "embodiedIn";

        recsPerFile = 50000;
        
        sampleExport = false;

        sampleFromIndex = 0;

        sampleToIndex = 1000;
    }

    /**
     * Stem for the export file name.
     * @return the fileStem
     */
    public String getFileStem() {
        return fileStem;
    }

    /**
     * Stem for the export file name.
     * @param fileStem the fileStem to set
     */
    public void setFileStem(String fileStem) {
        this.fileStem = fileStem;
    }

    /**
     * How many records to a file.
     * @return the recsPerFile
     */
    public int getRecsPerFile() {
        return recsPerFile;
    }

    /**
     * How many records to a file.
     * @param recsPerFile the recsPerFile to set
     */
    public void setRecsPerFile(int recsPerFile) {
        this.recsPerFile = recsPerFile;
    }

    /**
     * Is this a sample export.
     * @return the isSampleExport
     */
    public boolean isSampleExport() {
        return sampleExport;
    }

    /**
     * Is this a sample export.
     * @param isSampleExport the isSampleExport to set
     */
    public void setSampleExport(boolean sampleExport) {
        this.sampleExport = sampleExport;
    }

    /**
     * Sample from index, inclusive.
     * @return the sampleFromIndex
     */
    public int getSampleFromIndex() {
        return sampleFromIndex;
    }

    /**
     * Sample from index, inclusive.
     * @param sampleFromIndex the sampleFromIndex to set
     */
    public void setSampleFromIndex(int sampleFromIndex) {
        this.sampleFromIndex = sampleFromIndex;
    }

    /**
     * Sample to index, exclusive.
     * @return the sampleToIndex
     */
    public int getSampleToIndex() {
        return sampleToIndex;
    }

    /**
     * Sample to index, exclusive.
     * @param sampleToIndex the sampleToIndex to set
     */
    public void setSampleToIndex(int sampleToIndex) {
        this.sampleToIndex = sampleToIndex;
    }

}
