/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.lookup;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import org.openrdf.rio.ntriples.NTriplesParser;

public class LocalFileSubjectLookup {
    private static final Map<String, String> authority = new HashMap<String, String>();
    private static final String AUTHORITY_FILE = "authority/lcsh-20110104.nt";
    private static final String PERF_LABEL_URL = "<http://www.w3.org/2004/02/skos/core#prefLabel>";
    private static final String ALT_LABEL_URL = "<http://www.w3.org/2004/02/skos/core#altLabel>";

    static {
        try{
            InputStream in = ClassLoader.getSystemResourceAsStream(AUTHORITY_FILE);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String strLine;
            LCSHSingleStatementHandler sh = new LCSHSingleStatementHandler();
            while ((strLine = br.readLine()) != null)   {
                if(strLine.contains(PERF_LABEL_URL) || strLine.contains(ALT_LABEL_URL)) {
                    NTriplesParser ntp = new NTriplesParser();
                    ntp.setStatementHandler(sh);
                    ntp.parse(new StringReader(strLine), "");
                    authority.put(sh.subjectHeading, sh.lcshURL);
                }
            }
        in.close();
        }catch (Exception e){
            System.err.println("Error: " + e.getMessage());
        }
    }

    private LocalFileSubjectLookup() {}

    public static String lookup(String subjectHeading) {
        if(authority.containsKey(subjectHeading)) {
            return authority.get(subjectHeading);
        } else {
            return null;
        }
    }
}