/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.rdf.tripleWriter;

import java.util.List;
import org.apache.commons.lang.StringEscapeUtils;

public class RDFExportTripleGeneral implements RDFExportTriple {
    public enum ElementForm {
        VALUE,
        RESOURCE
    }

    private String element;
    private List<String> values;
    private String type;
    private ExportGroup exportGroup;
    private ElementForm elementForm;
    private String comment;

    public RDFExportTripleGeneral(String element, List<String> values, String type, ExportGroup exportGroup, ElementForm elementForm, String comment) {
        this.values = values;
        this.element = element;
        this.type = type;
        this.exportGroup = exportGroup;
        this.elementForm = elementForm;
        this.comment = comment;
    }

    @Override
    public ExportGroup getExportGroup() {
        return this.exportGroup;
    }

    @Override
    public String exportToElement() {
        String retValue = "";
        for(String value : values) {
            if(value == null || value.trim().isEmpty()) {
                continue;
            }
            retValue += "        <" + element;
            if(type != null && !type.isEmpty()) {
                retValue += " " + type;
            }
            switch(this.elementForm) {
                case VALUE:
                    retValue += ">" + StringEscapeUtils.escapeXml(value) + "</" + element + ">\n";
                    break;
                case RESOURCE:
                    retValue += " rdf:resource=\"" + StringEscapeUtils.escapeXml(value) + "\"/>\n";
                    break;
            }
        }
        if(values.size() > 0 && !retValue.trim().isEmpty() && comment != null && !comment.trim().isEmpty()) {
            retValue += "        <!--" + comment + "-->\n";
        }
        return retValue;
    }
}

