/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.VfrbrEntity;
import java.io.PrintWriter;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class AssocWriter {

    private static final Logger log = Logger.getLogger(AssocWriter.class);

    private static String INDENT_2 = "    ";

    /**
     *  Simple relation for 1.0.
     * @param xmlOut
     * @param offset
     * @param sourceURI
     * @param targetURI
     */
    public void writeSimple(PrintWriter xmlOut,
                            String offset,
                            String sourceURI,
                            String targetURI) {

        // sourceURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "source=\"" + sourceURI + "\"");

        // targetURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "target=\"" + targetURI + "\"");

        xmlOut.println("/>");
    }

    /**
     *  Simple relation for 1.1.
     * @param xmlOut
     * @param offset
     * @param sourceEntity
     * @param sourceURI
     * @param targetEntity
     * @param targetURI
     */
    public void writeSimple(PrintWriter xmlOut,
                            String offset,
                            String sourceEntity,
                            String sourceURI,
                            String targetEntity,
                            String targetURI) {

        // sourceEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceEntity=\"" + sourceEntity + "\"");

        // sourceURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceURI=\"" + sourceURI + "\"");

        // targetEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetEntity=\"" + targetEntity + "\"");

        // targetURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetURI=\"" + targetURI + "\"");

        xmlOut.println("/>");
    }

    public void writeRole(PrintWriter xmlOut,
                          String offset,
                          String sourceEntity,
                          String sourceURI,
                          String targetEntity,
                          String targetURI,
                          String role) {

        // sourceEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceEntity=\"" + sourceEntity + "\"");

        // sourceURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceURI=\"" + sourceURI + "\"");

        // targetEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetEntity=\"" + targetEntity + "\"");

        // targetURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetURI=\"" + targetURI + "\"");

        // role, optional
        if (role != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "role=\"" + role + "\"");
        }

        xmlOut.println("/>");
    }

    /**
     *  Categoried relation for 1.0
     * @param xmlOut
     * @param offset
     * @param source
     * @param target
     * @param category
     */
    public void writeCategoried(PrintWriter xmlOut,
                                String offset,
                                String sourceURI,
                                String targetURI,
                                String category) {

        // source, required
        xmlOut.print("\n" + offset + INDENT_2
                + "source=\"" + sourceURI + "\"");

        // target, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetURI=\"" + targetURI + "\"");

        // category attribute is optionsl
        if (category != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "category=\"" + category + "\"");
        }

        xmlOut.println("/>");

    }

    /**
     *  Categoried relation for 1.1
     * @param xmlOut
     * @param offset
     * @param sourceEntity
     * @param sourceURI
     * @param targetEntity
     * @param targetURI
     * @param category
     */
    public void writeCategoried(PrintWriter xmlOut,
                                String offset,
                                String sourceEntity,
                                String sourceURI,
                                String targetEntity,
                                String targetURI,
                                String category) {

        // sourceEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceEntity=\"" + sourceEntity + "\"");

        // sourceURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "sourceURI=\"" + sourceURI + "\"");

        // targetEntity, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetEntity=\"" + targetEntity + "\"");

        // targetURI, required
        xmlOut.print("\n" + offset + INDENT_2
                + "targetURI=\"" + targetURI + "\"");

        // category attribute is optionsl
        if (category != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "category=\"" + category + "\"");
        }

        xmlOut.println("/>");

    }

    public String entityType(VfrbrEntity vfrbrEnt) {
        String typeName = null;

        String className = vfrbrEnt.getClass().getSimpleName();

        if ("WorkJpa".equals(className)) {
            typeName = "work";

        } else if ("ExpressionJpa".equals(className)) {
            typeName = "expression";

        } else if ("ManifestationJpa".equals(className)) {
            typeName = "manifestation";

        } else if ("ItemJpa".equals(className)) {
            typeName = "item";

        } else if ("PersonJpa".equals(className)) {
            typeName = "person";

        } else if ("CorporateBodyJpa".equals(className)) {
            typeName = "corporateBody";

        } else if ("FamilyJpa".equals(className)) {
            typeName = "family";

        } else if ("ConceptJpa".equals(className)) {
            typeName = "concept";

        } else if ("EventJpa".equals(className)) {
            typeName = "event";

        } else if ("Object".equals(className)) {
            typeName = "object";

        } else if ("PlaceJpa".equals(className)) {
            typeName = "place";

        } // className not recognized
        else {
            System.out.println(""
                    + "****** AssocWriter.entityType: unrecognized className of: \""
                    + className
                    + "\"");
        }

        return typeName;
    }
}
