/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.persist.entity.attrib.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class AttribWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    /**
     *  Write a string.
     * Used to put the formOfExpression into Manififestation.notes
     * @param xmlOut
     * @param xmlWrite
     * @param offset
     * @param string
     * @throws XMLStreamException
     */
    public void writeString(PrintWriter xmlOut,
                            XMLStreamWriter xmlWrite,
                            String offset,
                            String string)
            throws XMLStreamException {

        xmlOut.println(">");

        if (string != null) {
            xmlOut.flush();

            xmlWrite.writeCharacters(offset + INDENT_1 + string);
            xmlWrite.flush();

            xmlOut.println();
        }
    }

    public void writeSimple(PrintWriter xmlOut,
                            XMLStreamWriter xmlWrite,
                            String offset,
                            AttribSimple attrib)
            throws XMLStreamException {

        xmlOut.println(">");

        if (attrib.getText() != null) {
            xmlOut.flush();

            xmlWrite.writeCharacters(offset + INDENT_1 + attrib.getText());
            xmlWrite.flush();

            xmlOut.println();
        }
    }

    public void writeJurisNormalVocab(PrintWriter xmlOut,
                                      XMLStreamWriter xmlWrite,
                                      String offset,
                                      AttribJurisNormalVocab attrib)
            throws XMLStreamException {

        if (attrib.getJurisdiction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "jurisdiction=\""
                    + attrib.getJurisdiction()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeJurisNormalVocabTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribJurisNormalVocabTime attrib)
            throws XMLStreamException {

        if (attrib.getJurisdiction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "jurisdiction=\""
                    + attrib.getJurisdiction()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeNormal(PrintWriter xmlOut,
                            XMLStreamWriter xmlWrite,
                            String offset,
                            AttribNormal attrib)
            throws XMLStreamException {

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeType(PrintWriter xmlOut,
                          XMLStreamWriter xmlWrite,
                          String offset,
                          AttribType attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeAvail(PrintWriter xmlOut,
                               XMLStreamWriter xmlWrite,
                               String offset,
                               AttribTypeAvail attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        /*
         * an error in schema release 1.0
         * has "avail" for availability in TypeAvail
         */
        if (attrib.getAvailability() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "avail=\""
                    + attrib.getAvailability()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeAvailTime(PrintWriter xmlOut,
                                   XMLStreamWriter xmlWrite,
                                   String offset,
                                   AttribTypeAvailTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getAvailability() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "availability=\""
                    + attrib.getAvailability()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeJurisNormalVocab(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeJurisNormalVocab attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getJurisdiction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "jurisdiction=\""
                    + attrib.getJurisdiction()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeJurisNormalVocabTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeJurisNormalVocabTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getJurisdiction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "jurisdiction=\""
                    + attrib.getJurisdiction()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeJurisVocab(PrintWriter xmlOut,
                                    XMLStreamWriter xmlWrite,
                                    String offset,
                                    AttribTypeJurisVocab attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getJurisdiction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "jurisdiction=\""
                    + attrib.getJurisdiction()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormal(PrintWriter xmlOut,
                                XMLStreamWriter xmlWrite,
                                String offset,
                                AttribTypeNormal attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormalFunction(PrintWriter xmlOut,
                                        XMLStreamWriter xmlWrite,
                                        String offset,
                                        AttribTypeNormalFunction attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getFunction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "function=\""
                    + attrib.getFunction()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormalVocab(PrintWriter xmlOut,
                                     XMLStreamWriter xmlWrite,
                                     String offset,
                                     AttribTypeNormalVocab attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormalVocabAvailTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeNormalVocabAvailTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getAvailability() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "availability=\""
                    + attrib.getAvailability()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormalVocabOffset(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeNormalVocabOffset attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getOffset() != 0) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "offset=\""
                    + attrib.getOffset()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeNormalVocabTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeNormalVocabTime attrib)
            throws XMLStreamException {

        writeTypeNormalVocabTime(xmlOut,
                                 xmlWrite,
                                 offset,
                                 attrib,
                                 false);
    }

    /*
     * This allows Person and CorporateBody
     * to use the normal attribute in the frbrization loading
     * for matching persisted instances
     * but not export normal to the xml output.
     */
    public void writeTypeNormalVocabTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeNormalVocabTime attrib,
            boolean skipNormal)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (!skipNormal) {
            if (attrib.getNormal() != null) {
                xmlOut.print("\n" + offset + INDENT_2
                        + "normal=\""
                        + attrib.getNormal()
                        + "\"");
            }
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeOffset(PrintWriter xmlOut,
                                XMLStreamWriter xmlWrite,
                                String offset,
                                AttribTypeOffset attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getOffset() != 0) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "offset=\""
                    + attrib.getOffset()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeTime(PrintWriter xmlOut,
                              XMLStreamWriter xmlWrite,
                              String offset,
                              AttribTypeTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeVocab(PrintWriter xmlOut,
                               XMLStreamWriter xmlWrite,
                               String offset,
                               AttribTypeVocab attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeVocabFunctionTime(
            PrintWriter xmlOut,
            XMLStreamWriter xmlWrite,
            String offset,
            AttribTypeVocabFunctionTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getFunction() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "function=\""
                    + attrib.getFunction()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeVocabOffset(PrintWriter xmlOut,
                                     XMLStreamWriter xmlWrite,
                                     String offset,
                                     AttribTypeVocabOffset attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getOffset() != 0) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "offset=\""
                    + attrib.getOffset()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeTypeVocabTime(PrintWriter xmlOut,
                                   XMLStreamWriter xmlWrite,
                                   String offset,
                                   AttribTypeVocabTime attrib)
            throws XMLStreamException {

        if (attrib.getType() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "type=\""
                    + attrib.getType()
                    + "\"");
        }

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeVocab(PrintWriter xmlOut,
                           XMLStreamWriter xmlWrite,
                           String offset,
                           AttribVocab attrib)
            throws XMLStreamException {

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeVocabNormal(PrintWriter xmlOut,
                                 XMLStreamWriter xmlWrite,
                                 String offset,
                                 AttribVocabNormal attrib)
            throws XMLStreamException {

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeVocabNormalTime(PrintWriter xmlOut,
                                     XMLStreamWriter xmlWrite,
                                     String offset,
                                     AttribVocabNormalTime attrib)
            throws XMLStreamException {

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getNormal() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "normal=\""
                    + attrib.getNormal()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeVocabQuant(PrintWriter xmlOut,
                                XMLStreamWriter xmlWrite,
                                String offset,
                                AttribVocabQuant attrib)
            throws XMLStreamException {

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getQuantity() != 0) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "quantity=\""
                    + attrib.getQuantity()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }

    public void writeVocabTime(PrintWriter xmlOut,
                               XMLStreamWriter xmlWrite,
                               String offset,
                               AttribVocabTime attrib)
            throws XMLStreamException {

        if (attrib.getVocabulary() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "vocabulary=\""
                    + attrib.getVocabulary()
                    + "\"");
        }

        if (attrib.getTimeframe() != null) {
            xmlOut.print("\n" + offset + INDENT_2
                    + "timeframe=\""
                    + attrib.getTimeframe()
                    + "\"");
        }

        this.writeSimple(xmlOut, xmlWrite, offset, attrib);
    }
}
