/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class CorporateBodyWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private static boolean skipNormal = true;

    private final AttribWriter attribWriter;

    public CorporateBodyWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEfrbrCorporateBody(PrintWriter xmlOut,
                                   XMLStreamWriter xmlWrite,
                                   CorporateBodyJpa corp)
            throws XMLStreamException {

        xmlOut.print(INDENT_2 + "<efrbr-corporateBody:corporateBody");

        xmlOut.print("\n" + INDENT_4
                + "identifier=\"" + XMLExportContext.makeURI("corporateBody", corp.getId())
                + "\"");

        xmlOut.println(">");

        xmlOut.println(INDENT_3 + "<!-- names:[" + corp.getNames().size() + "] -->");
        for (CorporateBodyName name : corp.getNames()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:nameOfTheCorporateBody");
            // TypeVocabTime
            attribWriter.writeTypeNormalVocabTime(xmlOut, xmlWrite, INDENT_3, name, skipNormal);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:nameOfTheCorporateBody>");
        }

        xmlOut.println(INDENT_3 + "<!-- numbers: [" + corp.getNumbers().size() + "] -->");
        for (CorporateBodyNumber number : corp.getNumbers()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:numberAssociatedWithTheCorporateBody");
            // TypeNormal
            attribWriter.writeTypeNormal(xmlOut, xmlWrite, INDENT_3, number);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:numberAssociatedWithTheCorporateBody>");
        }

        xmlOut.println(INDENT_3 + "<!-- places: [" + corp.getPlaces().size() + "] -->");
        for (CorporateBodyPlace place : corp.getPlaces()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:placeAssociatedWithTheCorporateBody");
            // JusisNormalVocabTime
            attribWriter.writeJurisNormalVocabTime(xmlOut, xmlWrite, INDENT_3, place);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:placeAssociatedWithTheCorporateBody>");
        }

        xmlOut.println(INDENT_3 + "<!-- dates: [" + corp.getDates().size() + "] -->");
        for (CorporateBodyDate date : corp.getDates()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:dateAssociatedWithTheCorporateBody");
            // TypeNormalFunction
            attribWriter.writeTypeNormalFunction(xmlOut, xmlWrite, INDENT_3, date);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:dateAssociatedWithTheCorporateBody>");
        }

        xmlOut.println(INDENT_3 + "<!-- designations: [" + corp.getDesignations().size() + "] -->");
        for (CorporateBodyDesignation desig : corp.getDesignations()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:otherDesignationAssociatedWithTheCorporateBody");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, desig);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:otherDesignationAssociatedWithTheCorporateBody>");
        }

//            <!-- FRAD attributes -->

        xmlOut.println(INDENT_3 + "<!-- languages: [" + corp.getLanguages().size() + "] -->");
        for (CorporateBodyLanguage lang : corp.getLanguages()) {
            xmlOut.print(INDENT_3 + "<efrad-corporateBody:languageOfTheCorporateBody");
            // VocabNormalTime
            attribWriter.writeVocabNormalTime(xmlOut, xmlWrite, INDENT_3, lang);
            xmlOut.println(INDENT_3 + "</efrad-corporateBody:languageOfTheCorporateBody>");
        }

        xmlOut.println(INDENT_3 + "<!-- languages: [" + corp.getAddresses().size() + "] -->");
        for (CorporateBodyAddress addr : corp.getAddresses()) {
            xmlOut.print(INDENT_3 + "<efrad-corporateBody:address");
            // TypeAvailTime
            attribWriter.writeTypeAvailTime(xmlOut, xmlWrite, INDENT_3, addr);
            xmlOut.println(INDENT_3 + "</efrad-corporateBody:address>");
        }

        xmlOut.println(INDENT_3 + "<!-- activities: [" + corp.getActivityFields().size() + "] -->");
        for (CorporateBodyActivityField activ : corp.getActivityFields()) {
            xmlOut.print(INDENT_3 + "<efrad-corporateBody:fieldOfActivity");
            // VocabNormalTime
            attribWriter.writeVocabNormalTime(xmlOut, xmlWrite, INDENT_3, activ);
            xmlOut.println(INDENT_3 + "</efrad-corporateBody:fieldOfActivity>");
        }

        xmlOut.println(INDENT_3 + "<!-- histories: [" + corp.getHistories().size() + "] -->");
        for (CorporateBodyHistory hist : corp.getHistories()) {
            xmlOut.print(INDENT_3 + "<efrad-corporateBody:history");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, hist);
            xmlOut.println(INDENT_3 + "</efrad-corporateBody:history>");
        }

//            <!--
//              extensions to entity attributes
//              -->

        xmlOut.println(INDENT_3 + "<!-- notes:[" + corp.getNotes().size() + "] -->");
        for (CorporateBodyNote note : corp.getNotes()) {
            xmlOut.print(INDENT_3 + "<efrbr-corporateBody:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);
            xmlOut.println(INDENT_3 + "</efrbr-corporateBody:note>");
        }

        xmlOut.println(INDENT_2 + "</efrbr-corporateBody:corporateBody>");
    }
}
