/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.subject.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class EventWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;

    public EventWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEvent(PrintWriter xmlOut,
                           XMLStreamWriter xmlWrite,
                                   XMLExportContext exportContext,
                           EventJpa event)
            throws XMLStreamException {
        //
//      <xsd:element name="event">

        xmlOut.print(INDENT_2 + "<vfrbr-event:event");

//      <xsd:attribute name="identifier"
//                     type="xsd:anyURI"
//                     use="optional"/>

//        if (event.getURI() != null) {
//            xmlOut.print("\n" + INDENT_2 + INDENT_2
//                    + "identifier=\"" + event.getURI() + "\"");
//        }
        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + exportContext.makeURI("Event", event.getId())
                + "\"");

        xmlOut.println(">");

//        <xsd:complexType>
//          <xsd:sequence>
//            <xsd:element ref="termForTheEvent"
//                         minOccurs="0" maxOccurs="unbounded"/>

        xmlOut.println(INDENT_3 + "<!-- terms: ["
                + event.getTerms().size() + "] -->");
        for (EventTerm term : event.getTerms()) {
            xmlOut.print(INDENT_3 + "<vfrbr-event:termForTheEvent");
            // TypeVocab
            attribWriter.writeTypeVocab(xmlOut, xmlWrite, INDENT_3, term);

            xmlOut.println(INDENT_3 + "</vfrbr-event:termForTheEvent>");
        }

//            <!--
//              extensions to entity attributes
//              -->
//            <xsd:element ref="note"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- notes: ["
                + event.getNotes().size() + "] -->");
        for (EventNote note : event.getNotes()) {
            xmlOut.print(INDENT_3 + "<vfrbr-event:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);

            xmlOut.println(INDENT_3 + "</vfrbr-event:note>");
        }

//            <xsd:element ref="extensions"
//                         minOccurs="0" maxOccurs="1"/>
//          </xsd:sequence>
//          <!-- entity element identifier attribute -->
//          <xsd:attribute name="identifier"
//                         type="xsd:anyURI"
//                         use="optional"/>
//        </xsd:complexType>
//      </xsd:element>

        xmlOut.println(INDENT_2 + "</vfrbr-event:event>");
    }
}
