/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.expression.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class ExpressionWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;

    public ExpressionWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEfrbrExpression(PrintWriter xmlOut,
                                XMLStreamWriter xmlWrite,
                                ExpressionJpa expr)
            throws XMLStreamException {

        xmlOut.print(INDENT_2 + "<efrbr-expression:expression");

        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + XMLExportContext.makeURI("expression", expr.getId())
                + "\"");
        xmlOut.println(">");

        xmlOut.println(INDENT_3 + "<!-- titles: [" + expr.getTitles().size() + "] -->");
        for (ExpressionTitle title : expr.getTitles()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:titleOfTheExpression");
            // TypeVocabOffset
            attribWriter.writeTypeVocabOffset(xmlOut, xmlWrite, INDENT_3, title);
            xmlOut.println(INDENT_3 + "</efrbr-expression:titleOfTheExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- forms: [" + expr.getForms().size() + "] -->");
        for (ExpressionForm form : expr.getForms()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:formOfExpression");
            // Vocab
            attribWriter.writeVocab(xmlOut, xmlWrite, INDENT_3, form);
            xmlOut.println(INDENT_3 + "</efrbr-expression:formOfExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- dates: [" + expr.getDates().size() + "] -->");
        for (ExpressionDate date : expr.getDates()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:dateOfExpression");
            // TypeNormal
            attribWriter.writeTypeNormal(xmlOut, xmlWrite, INDENT_3, date);
            xmlOut.println(INDENT_3 + "</efrbr-expression:dateOfExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- languages: [" + expr.getLanguages().size() + "] -->");
        for (ExpressionLanguage lang : expr.getLanguages()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:languageOfExpression");
            // VocabNormal
            attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3, lang);
            xmlOut.println(INDENT_3 + "</efrbr-expression:languageOfExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- characteristics: [" + expr.getCharacteristics().size() + "] -->");
        for (ExpressionCharacteristic charac : expr.getCharacteristics()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:otherDistinguishingCharacteristic");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, charac);
            xmlOut.println(INDENT_3 + "</efrbr-expression:otherDistinguishingCharacteristic>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="extensibilityOfExpression"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="revisabilityOfExpression"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

        xmlOut.println(INDENT_3 + "<!-- extents: [" + expr.getExtents().size() + "] -->");
        for (ExpressionExtent extent : expr.getExtents()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:extentOfTheExpression");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, extent);
            xmlOut.println(INDENT_3 + "</efrbr-expression:extentOfTheExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- summarizations: [" + expr.getSummarizations().size() + "] -->");
        for (ExpressionSummarization summar : expr.getSummarizations()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:summarizationOfContent");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, summar);
            xmlOut.println(INDENT_3 + "</efrbr-expression:sumarizationOfContent>");
        }

        xmlOut.println(INDENT_3 + "<!-- contexts: [" + expr.getContexts().size() + "] -->");
        for (ExpressionContext context : expr.getContexts()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:contextForTheExpression");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, context);
            xmlOut.println(INDENT_3 + "</efrbr-expression:contextForTheExpression>");
        }

        xmlOut.println(INDENT_3 + "<!-- responses: [" + expr.getResponses().size() + "] -->");
        for (ExpressionResponse resp : expr.getResponses()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:criticalResponseToTheExpression");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, resp);
            xmlOut.println(INDENT_3 + "</efrbr-expression:criticalResponseToTheExpression>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="useRestrictionsOnTheExpression"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="sequencingPattern"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="expectedRegularityOfIssue"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="expectedFrequencyOfIssue"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

//            <xsd:element ref="typeOfScore"
//                         minOccurs="0" maxOccurs="1"
//                         />
        xmlOut.println(INDENT_3 + "<!-- scoreTypes: [" + expr.getScoreType().size() + "] -->");
        for (ExpressionScoreType scoreType : expr.getScoreType()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:typeOfScore");
            // Vocab
            attribWriter.writeVocab(xmlOut, xmlWrite, INDENT_3, scoreType);
            xmlOut.println(INDENT_3 + "</efrbr-expression:typeOfScore>");
        }

        xmlOut.println(INDENT_3 + "<!-- performanceMediums: [" + expr.getPerformanceMediums().size() + "] -->");
        for (ExpressionPerformanceMedium perfMed : expr.getPerformanceMediums()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:mediumOfPerformance");
            // VocabQuant
            attribWriter.writeVocabQuant(xmlOut, xmlWrite, INDENT_3, perfMed);
            xmlOut.println(INDENT_3 + "</efrbr-expression:mediumOfPerformance>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="scale"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="projection"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="presentationTechnique"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="representationOfRelief"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="geodeticGridAndVerticalMeasurement"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="recordingTechnique"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="specialCharacteristic"
//              minOccurs="0" maxOccurs="unbounded"/>
//            <xsd:element
//              ref="technique"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

//            <!--
//              extensions to entity attributes
//              -->

        xmlOut.println(INDENT_3 + "<!-- notes: [" + expr.getNotes().size() + "] -->");
        for (ExpressionNote note : expr.getNotes()) {
            xmlOut.print(INDENT_3 + "<efrbr-expression:note");
            // TypeVail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);
            xmlOut.println(INDENT_3 + "</efrbr-expression:note>");
        }

        xmlOut.println(INDENT_2 + "</efrbr-expression:expression>");

    }
}
