/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class FamilyWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;

    public FamilyWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeFamily(PrintWriter xmlOut,
                            XMLStreamWriter xmlWrite,
                                   XMLExportContext exportContext,
                            FamilyJpa family)
            throws XMLStreamException {
        //

//      <xsd:element name="family">

        xmlOut.print(INDENT_2 + "<vfrad-family:family");

//          <xsd:attribute name="identifier"
//                         type="xsd:anyURI"
//                         use="optional"/>

//        if (family.getURI() != null) {
//            xmlOut.print("\n" + INDENT_2 + INDENT_2
//                    + "identifier=\"" + family.getURI() + "\"");
//        }
        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + exportContext.makeURI("Family", family.getId())
                + "\"");

        xmlOut.println(">");


//        <xsd:complexType>
//          <xsd:sequence>
//            <!-- FRAD attributes -->

//            <xsd:element ref="typeOfFamily"
//                         minOccurs="0" maxOccurs="1"
//                         />
        // OneToOne
        xmlOut.print(INDENT_3 + "<vfrad-family:typeOfFamily");
        // VocabNormal
        attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_3,
                                      family.getType());

        xmlOut.println(INDENT_3 + "</vfrad-family:typeOfFamily>");

//            <xsd:element ref="datesOfFamily"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- dates: ["
                + family.getDates().size() + "] -->");
        for (FamilyDate date : family.getDates()) {
            xmlOut.print(INDENT_3 + "<vfrad-family:datesOfFamily");
            // TypeNormal
            attribWriter.writeTypeNormal(xmlOut, xmlWrite, INDENT_3, date);

            xmlOut.println(INDENT_3 + "</vfrad-family:datesOfFamily>");
        }

//            <xsd:element ref="placesAssociatedWithFamily"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- places: ["
                + family.getPlaces().size() + "] -->");
        for (FamilyPlace place : family.getPlaces()) {
            xmlOut.print(INDENT_3 + "<vfrad-family:placesAssociatedWithFamily");
            // TypeJurisNormalVocabTime
            attribWriter.writeTypeJurisNormalVocabTime(xmlOut, xmlWrite,
                                                       INDENT_3, place);

            xmlOut.println(INDENT_3
                    + "</vfrad-family:placesAssociatedWithFamily>");
        }

//            <xsd:element ref="fieldOfActivity"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- activityFields: ["
                + family.getActivityFields().size() + "] -->");
        for (FamilyActivityField field : family.getActivityFields()) {
            xmlOut.print(INDENT_3 + "<vfrad-family:fieldOfActivity");
            // TypeNormalVocabTime
            attribWriter.writeTypeNormalVocabTime(xmlOut, xmlWrite, INDENT_3,
                                                  field);

            xmlOut.println(INDENT_3 + "</vfrad-family:fieldOfActivity>");
        }

//            <xsd:element ref="historyOfFamily"
//                         minOccurs="0" maxOccurs="unbounded"/>

        xmlOut.println(INDENT_3 + "<!-- histories: ["
                + family.getHistories().size() + "] -->");
        for (FamilyHistory history : family.getHistories()) {
            xmlOut.print(INDENT_3 + "<vfrad-family:historyOfFamily");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, history);

            xmlOut.println(INDENT_3 + "</vfrad-family:historyOfFamily>");
        }

//            <!--
//              extensions to entity attributes
//              -->
//            <xsd:element ref="vfrbr-family:note"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- notes: ["
                + family.getNotes().size() + "] -->");
        for (FamilyNote note : family.getNotes()) {
            xmlOut.print(INDENT_3 + "<vfrbr-family:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);

            xmlOut.println(INDENT_3 + "</vfrbr-family:note>");
        }

//            <xsd:element ref="vfrbr-family:extensions"
//                         minOccurs="0" maxOccurs="1"/>

//              <xsd:element name="extensions">
//                <xsd:complexType>
//                  <xsd:sequence>
//                    <xsd:element name="variations">
//                      <xsd:complexType>
//                        <xsd:sequence>
//                          <!-- no vFRBR Family attribute extensions -->
//                        </xsd:sequence>
//                      </xsd:complexType>
//                    </xsd:element>
//                  </xsd:sequence>
//                </xsd:complexType>
//              </xsd:element>

//          </xsd:sequence>
//          <!-- entity element identifier attribute -->
//          <xsd:attribute name="identifier"
//                         type="xsd:anyURI"
//                         use="optional"/>
//        </xsd:complexType>
//      </xsd:element>

        xmlOut.println(INDENT_2 + "</vfrad-family:family>");
    }
}
