/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.item.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class ItemWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;

    public ItemWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeItem(PrintWriter xmlOut,
                          XMLStreamWriter xmlWrite,
                                   XMLExportContext exportContext,
                          ItemJpa item)
            throws XMLStreamException {

//      <xsd:element name="item">

        xmlOut.print(INDENT_2 + "<vfrbr-item:item");

//          <xsd:attribute name="identifier"
//                         type="xsd:anyURI"
//                         use="optional"/>

//        if (item.getURI() != null) {
//            xmlOut.print("\n" + INDENT_2 + INDENT_2
//                    + "identifier=\"" + item.getURI() + "\"");
//        }
        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + exportContext.makeURI("Item", item.getId())
                + "\"");

        xmlOut.println(">");


//        <xsd:complexType>
//          <xsd:sequence>
//            <xsd:element ref="itemIdentifier"
//                         minOccurs="0" maxOccurs="unbounded"/>

        xmlOut.println(INDENT_3 + "<!-- identifiers: ["
                + item.getIdentifiers().size() + "] -->");
        for (ItemIdentifier ident : item.getIdentifiers()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:itemIdentifier");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, ident);

            xmlOut.println(INDENT_3 + "</vfrbr-item:itemIdentifier>");
        }

//            <!-- excluded
//            <xsd:element
//              ref="fingerprint"
//              minOccurs="0" maxOccurs="unbounded"/>
//              -->

//            <xsd:element ref="provenanceOfTheItem"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- provenances: ["
                + item.getProvenances().size() + "] -->");
        for (ItemProvenance prov : item.getProvenances()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:provenanceOfTheItem");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, prov);

            xmlOut.println(INDENT_3 + "</vfrbr-item:provenanceOfTheItem>");
        }

//            <xsd:element ref="marksInscriptions"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- marks: ["
                + item.getMarks().size() + "] -->");
        for (ItemMark mark : item.getMarks()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:marksInscriptions");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, mark);

            xmlOut.println(INDENT_3 + "</vfrbr-item:marksInscriptions>");
        }

//            <xsd:element ref="exhibitionHistory"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- exhibitions: ["
                + item.getExhibitions().size() + "] -->");
        for (ItemExhibitionHistory exhib : item.getExhibitions()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:exhibitionHistory");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, exhib);

            xmlOut.println(INDENT_3 + "</vfrbr-item:exhibitionHistory>");
        }

//            <xsd:element ref="conditionOfTheItem"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- conditions: ["
                + item.getConditions().size() + "] -->");
        for (ItemCondition cond : item.getConditions()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:conditionOfTheItem");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, cond);

            xmlOut.println(INDENT_3 + "</vfrbr-item:conditionOfTheItem>");
        }

//            <xsd:element ref="treatmentHistory"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- treatmentHistories: ["
                + item.getTreatmentHistories().size() + "] -->");
        for (ItemTreatmentHistory hist : item.getTreatmentHistories()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:treatmentHistory");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, hist);

            xmlOut.println(INDENT_3 + "</vfrbr-item:treatmentHistory>");
        }

//            <xsd:element ref="scheduledTreatment"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- treatmentSchedules: ["
                + item.getTreatmentSchedules().size() + "] -->");
        for (ItemTreatmentSchedule sched : item.getTreatmentSchedules()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:scheduledTreatment");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, sched);

            xmlOut.println(INDENT_3 + "</vfrbr-item:scheduledTreatment>");
        }

//            <xsd:element ref="accessRestrictionsOnTheItem"
//                         minOccurs="0" maxOccurs="unbounded"/>

        xmlOut.println(INDENT_3 + "<!-- accessRestrictions: ["
                + item.getAccessRestrictions().size() + "] -->");
        for (ItemAccessRestriction restr : item.getAccessRestrictions()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:accessRestrictionsOnTheItem");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, restr);

            xmlOut.println(INDENT_3
                    + "</vfrbr-item:accessRestrictionsOnTheItem>");
        }

//            <!-- FRAD attributes -->
//            <xsd:element ref="vfrad-item:locationOfItem"
//                         minOccurs="0" maxOccurs="1"
//                         />
        // OneToOne
        xmlOut.print(INDENT_3 + "<vfrad-item:locationOfItem");
        // TypeNormalVocabAvailTime
        attribWriter.writeTypeNormalVocabAvailTime(xmlOut, xmlWrite, INDENT_3, item.
                getLocation());

        xmlOut.print(INDENT_3 + "</vfrad-item:locationOfItem>");

//            <!-- excluded
//            <xsd:element ref="vfrad-item:custodialHistoryOfItem"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />
//              -->

//            <xsd:element ref="vfrad-item:immediateSourceOfAcquisitionOfItem"
//                         minOccurs="0" maxOccurs="unbounded"/>

        xmlOut.println(INDENT_3 + "<!-- acquisitionSources: ["
                + item.getAcquisitionSources().size() + "] -->");
        for (ItemAcquisitionSource source : item.getAcquisitionSources()) {
            xmlOut.print(INDENT_3
                    + "<vfrad-item:immediateSourceOfAcquisitionOfItem");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, source);

            xmlOut.println(INDENT_3
                    + "</vfrad-item:immediateSourceOfAcquisitionOfItem>");
        }

//            <!--
//              extensions to entity attributes
//              -->
//            <xsd:element ref="note"
//                         minOccurs="0" maxOccurs="unbounded"
//                         />

        xmlOut.println(INDENT_3 + "<!-- notes: ["
                + item.getNotes().size() + "] -->");
        for (ItemNote note : item.getNotes()) {
            xmlOut.print(INDENT_3 + "<vfrbr-item:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);

            xmlOut.print(INDENT_3 + "</vfrbr-item:note>");
        }

//            <xsd:element ref="extensions"
//                         minOccurs="0" maxOccurs="1"/>

//              <xsd:element name="extensions">

        xmlOut.println(INDENT_3 + "<vfrbr-item:extensions>");

//                <xsd:complexType>
//                  <xsd:sequence>
//                    <xsd:element name="variations">

        xmlOut.println(INDENT_4 + "<vfrbr-item:variations>");

//                      <xsd:complexType>
//                        <xsd:sequence>
//                          <xsd:element ref="callNumber"
//                                       minOccurs="0" maxOccurs="1"
//                                       />
        // OneToOne
        xmlOut.print(INDENT_5 + "<vfrbr-item:callNumber");
        // VocabNormal
        attribWriter.writeVocabNormal(xmlOut, xmlWrite, INDENT_5, item.
                getCallNumber());

        xmlOut.println(INDENT_5 + "</vfrbr-item:callNumber>");

//                          <xsd:element ref="copyNumber"
//                                       minOccurs="0" maxOccurs="1"/>
        // OneToOne
        xmlOut.print(INDENT_5 + "<vfrbr-item:copyNumber");
        // Simple
        attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_5,
                                 item.getCopyNumber());

        xmlOut.println(INDENT_5 + "</vfrbr-item:copyNumber>");

//                        </xsd:sequence>
//                      </xsd:complexType>
//                    </xsd:element>

        xmlOut.println(INDENT_4 + "</vfrbr-item:variations>");

//                  </xsd:sequence>
//                </xsd:complexType>
//              </xsd:element>

        xmlOut.println(INDENT_3 + "</vfrbr-item:extensions>");

//          </xsd:sequence>
//          <!-- entity element identifier attribute -->
//          <xsd:attribute name="identifier"
//                         type="xsd:anyURI"
//                         use="optional"/>
//        </xsd:complexType>
//      </xsd:element>

    }
}
