/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.relation.CorporateBodyToCorporateBody;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToExpression;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToWork;
import edu.indiana.dlib.vfrbr.persist.relation.FamilyToFamily;
import edu.indiana.dlib.vfrbr.persist.relation.FamilyToPerson;
import edu.indiana.dlib.vfrbr.persist.relation.ItemToItem;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToManifestation;
import edu.indiana.dlib.vfrbr.persist.relation.PersonToPerson;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToWork;
import java.io.PrintWriter;

/**
 *
 * @author pbmcelwa
 */
public class OtherWriter {

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private final AssocWriter assocWriter;


    public OtherWriter() {
        this.assocWriter = new AssocWriter();
    }

    public void writeOtherStart(PrintWriter xmlOut) {

        xmlOut.println(INDENT_2 + "<vfrbr-other:otherRelations>");
    }

    public void writeOtherEnd(PrintWriter xmlOut) {

        xmlOut.println(INDENT_2 + "</vfrbr-other:otherRelations>");
    }

//        <xsd:element ref="hasPart"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasPartWork(PrintWriter xmlOut,
                                     WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasPart");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

    public void writeExprHasPartExpr(PrintWriter xmlOut,
                                     ExpressionToExpression assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasPart");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getTargetExpression().getId()),
                assoc.getRelCategory());
    }

    public void writeManifHasPartManif(PrintWriter xmlOut,
                                       ManifestationToManifestation assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasPart");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "manifestation",
                XMLExportContext.makeURI("Manifestation",
                                   assoc.getSourceManifestation().getId()),
                "manifestation",
                XMLExportContext.makeURI("Manifestation",
                                   assoc.getTargetManifestation().getId()),
                assoc.getRelCategory());
    }

    public void writeItemHasPartItem(PrintWriter xmlOut,
                                     ItemToItem assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasPart");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "item",
                XMLExportContext.makeURI("Item",
                                   assoc.getSourceItem().getId()),
                "item",
                XMLExportContext.makeURI("Item",
                                   assoc.getTargetItem().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="vfrad-other:hasPart"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeCorpHasPartCorp(PrintWriter xmlOut,
                                         CorporateBodyToCorporateBody assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasPart");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "corporateBody",
                XMLExportContext.makeURI("CorporateBody",
                                   assoc.getSourceCorporateBody().getId()),
                "corporateBody",
                XMLExportContext.makeURI("CorporateBody",
                                   assoc.getTargetCorporateBody().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasSuccessor"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasSuccesorWork(PrintWriter xmlOut,
                                         WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasSuccessor");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

    public void writeExprHasSuccessorWork(PrintWriter xmlOut,
                                          ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasSuccessor");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "Work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasSupplement"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeExprHasSupplementWork(PrintWriter xmlOut,
                                           ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasSupplement");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasComplement"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeExprHasComplementWork(PrintWriter xmlOut,
                                           ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasComplement");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasSummary"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeExprHasSummaryWork(PrintWriter xmlOut,
                                        ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasSummary");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasAdaptation"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasAdaptationWork(PrintWriter xmlOut,
                                           WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasAdaptation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

    public void writeExprHasAdaptationWork(PrintWriter xmlOut,
                                           ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasAdaptation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasTransformation"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeExprHasTransformationWork(PrintWriter xmlOut,
                                               ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasTransformation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasImitation"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasImitationWork(PrintWriter xmlOut,
                                          WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasImitation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

    public void writeExprHasImitationWork(PrintWriter xmlOut,
                                          ExpressionToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasImitation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()),
                assoc.getRelCategory());
    }

    public void writeExprHasImitationExpr(PrintWriter xmlOut,
                                          ExpressionToExpression assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasImitation");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getTargetExpression().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="vfrad-other:hasEquivalent"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasEquivalentWork(PrintWriter xmlOut,
                                               WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasEquivalent");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

//        <xsd:element ref="vfrad-other:hasDerivative"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasDerivativeWork(PrintWriter xmlOut,
                                               WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasDerivative");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

//        <xsd:element ref="vfrad-other:hasDescriptive"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasDescriptiveWork(PrintWriter xmlOut,
                                                WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasDescriptive");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

//        <xsd:element ref="vfrad-other:hasSequence"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasSequenceWork(PrintWriter xmlOut,
                                             WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasSequence");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

    public void writeCorpHasSequenceCorp(PrintWriter xmlOut,
                                             CorporateBodyToCorporateBody assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasSequence");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "corporateBody",
                XMLExportContext.makeURI("CorporateBody",
                                   assoc.getSourceCorporateBody().getId()),
                "corporateBody",
                XMLExportContext.makeURI("CorporateBody",
                                   assoc.getTargetCorporateBody().getId()));
    }

//        <xsd:element ref="vfrad-other:hasSharedCharacteristic"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeWorkHasSharedCharacteristicWork(PrintWriter xmlOut,
                                                         WorkToWork assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasSharedCharacteristic");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                XMLExportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

    // excluded ...
//        <!--xsd:element ref="hasAbridgement"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
//        <xsd:element ref="hasRevision"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
//        <xsd:element ref="hasTranslation"
//                     minOccurs="0" maxOccurs="unbounded"
//                     / -->
    // ...
    //
//        <xsd:element ref="hasArrangement"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeExprHasArrangementExpr(PrintWriter xmlOut,
                                            ExpressionToExpression assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasArrangement");
        assocWriter.writeCategoried(
                xmlOut, INDENT_3,
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getSourceExpression().getId()),
                "expression",
                XMLExportContext.makeURI("Expression",
                                   assoc.getTargetExpression().getId()),
                assoc.getRelCategory());
    }

//        <xsd:element ref="hasReproduction"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeItemHasReproductionItem(PrintWriter xmlOut,
                                             ItemToItem assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasReproduction");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "item",
                XMLExportContext.makeURI("Item",
                                   assoc.getSourceItem().getId()),
                "item",
                XMLExportContext.makeURI("Item",
                                   assoc.getTargetItem().getId()));
    }

//        <xsd:element ref="hasAlternate"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeManifHasAlternateManif(PrintWriter xmlOut,
                                            ManifestationToManifestation assoc) {
        xmlOut.print(INDENT_3 + "<vfrbr-other:hasAlternate");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "manifestation",
                XMLExportContext.makeURI("Manifestation",
                                   assoc.getSourceManifestation().getId()),
                "manifestation",
                XMLExportContext.makeURI("Manifestation",
                                   assoc.getTargetManifestation().getId()));
    }

    // excluded ...
//        <!--xsd:element ref="hasReconfiguration"
//                     minOccurs="0" maxOccurs="unbounded"
//                     / -->
    // ...
    //
//        <xsd:element ref="vfrad-other:hasPseudonym"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasPseudonymPerson(PrintWriter xmlOut,
                                              PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasPseudonym");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasSecularName"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasSecularNamePerson(PrintWriter xmlOut,
                                                PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasSecularName");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasReligionName"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasReligionNamePerson(PrintWriter xmlOut,
                                                 PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasReligionName");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasOfficialName"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasOfficialNamePerson(PrintWriter xmlOut,
                                                 PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasOfficialName");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasAttribution"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasAttributionPerson(PrintWriter xmlOut,
                                                PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasAttribution");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasCollaboration"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasCollaborationPerson(PrintWriter xmlOut,
                                                  PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasCollaboration");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasSibling"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasSiblingPerson(PrintWriter xmlOut,
                                            PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasSibling");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasChild"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writePersonHasChildPerson(PrintWriter xmlOut,
                                          PersonToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasChild");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getSourcePerson().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }

//        <xsd:element ref="vfrad-other:hasMember"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />
    public void writeFamilyHasMemberPerson(PrintWriter xmlOut,
                                           FamilyToPerson assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasMember");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "family",
                XMLExportContext.makeURI("Family",
                                   assoc.getSourceFamily().getId()),
                "person",
                XMLExportContext.makeURI("Person",
                                   assoc.getTargetPerson().getId()));
    }
//        <xsd:element ref="vfrad-other:hasDescendent"
//                     minOccurs="0" maxOccurs="unbounded"
//                     />

    public void writeFamilyHasDescendentFamily(PrintWriter xmlOut,
                                               FamilyToFamily assoc) {
        xmlOut.print(INDENT_3 + "<vfrad-other:hasDescendent");
        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "family",
                XMLExportContext.makeURI("Family",
                                   assoc.getSourceFamily().getId()),
                "family",
                XMLExportContext.makeURI("Family",
                                   assoc.getTargetFamily().getId()));
    }
}
