/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.responsibleparty.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 *
 * @author pbmcelwa
 */
public class PersonWriter {

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private static boolean skipNormal = true;

    private final AttribWriter attribWriter;

    public PersonWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEfrbrPerson(PrintWriter xmlOut,
                            XMLStreamWriter xmlWrite,
                            PersonJpa person)
            throws XMLStreamException {

        xmlOut.print(INDENT_2 + "<efrbr-person:person");

        xmlOut.print("\n" + INDENT_4
                + "identifier=\"" + XMLExportContext.makeURI("person", person.getId())
                + "\"");

        xmlOut.println(">");

        xmlOut.println("<!-- names: [" + person.getNames().size() + "] -->");
        for (PersonName name : person.getNames()) {
            xmlOut.print(INDENT_3 + "<efrbr-person:nameOfPerson");
            // TypeVocabTime
            attribWriter.writeTypeNormalVocabTime(xmlOut, xmlWrite, INDENT_3, name, skipNormal);
            xmlOut.println(INDENT_3 + "</efrbr-person:nameOfPerson>");
        }

        xmlOut.println(INDENT_3 + "<!-- dates: [" + person.getDates().size() + "] -->");
        for (PersonDate date : person.getDates()) {
            xmlOut.print(INDENT_3 + "<efrbr-person:datesOfPerson");
            // TypeNormalFunction
            attribWriter.writeTypeNormalFunction(xmlOut, xmlWrite, INDENT_3, date);
            xmlOut.println(INDENT_3 + "</efrbr-person:datesOfPerson>");
        }

        xmlOut.println(INDENT_3 + "<!-- titles: [" + person.getTitles().size() + "] -->");
        for (PersonTitle title : person.getTitles()) {
            xmlOut.print(INDENT_3 + "<efrbr-person:titleOfPerson");
            // TypeTime
            attribWriter.writeTypeTime(xmlOut, xmlWrite, INDENT_3, title);
            xmlOut.println(INDENT_3 + "</efrbr-person:titleOfPerson>");
        }

        xmlOut.println(INDENT_3 + "<!-- designations: [" + person.getDesignations().size() + "] -->");
        for (PersonDesignation desig : person.getDesignations()) {
            xmlOut.print(INDENT_3 + "<efrbr-person:otherDesignationAssociatedWithThePerson");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, desig);
            xmlOut.println(INDENT_3 + "</efrbr-person:otherDesignationAssociatedWithThePerson>");
        }

//            <!-- FRAD attributes -->

        xmlOut.println(INDENT_3 + "<!-- genders: [" + person.getGenders().size() + "] -->");
        for (PersonGender gender : person.getGenders()) {
            xmlOut.print(INDENT_3 + "<efrad-person:gender");
            // VocabTime
            attribWriter.writeVocabTime(xmlOut, xmlWrite, INDENT_3, gender);
            xmlOut.println(INDENT_3 + "</efrad-person:gender>");
        }

        xmlOut.println(INDENT_3 + "<!-- birthPlaces: [" + person.getBirthPlaces().size() + "] -->");
        for (PersonBirthPlace birthPlace : person.getBirthPlaces()) {
            xmlOut.print(INDENT_3 + "<efrad-person:placeOfBirth");
            // JurisNormalVocab
            attribWriter.writeJurisNormalVocab(xmlOut, xmlWrite, INDENT_3, birthPlace);
            xmlOut.println(INDENT_3 + "</efrad-person:placeOfBirth>");
        }

        xmlOut.println(INDENT_3 + "<!-- deathPlaces: [" + person.getDeathPlaces().size() + "] -->");
        for (PersonDeathPlace deathPlace : person.getDeathPlaces()) {
            xmlOut.print(INDENT_3 + "<efrad-person:placeOfDeath");
            // JurisNormalVocab
            attribWriter.writeJurisNormalVocab(xmlOut, xmlWrite, INDENT_3, deathPlace);
            xmlOut.println(INDENT_3 + "</efrad-person:placeOfDeath>");
        }

        xmlOut.println(INDENT_3 + "<!-- countries: [" + person.getCountries().size() + "] -->");
        for (PersonCountry country : person.getCountries()) {
            xmlOut.print(INDENT_3 + "<efrad-person:country");
            // TypeNormalVocab
            attribWriter.writeTypeNormalVocab(xmlOut, xmlWrite, INDENT_3, country);
            xmlOut.println(INDENT_3 + "</efrad-person:country>");
        }

        xmlOut.println(INDENT_3 + "<!-- residencePlaces: [" + person.getResidencePlaces().size() + "] -->");
        for (PersonResidencePlace place : person.getResidencePlaces()) {
            xmlOut.print(INDENT_3 + "<efrad-person:placeOfResidence");
            // VocabNormalTime
            attribWriter.writeVocabNormalTime(xmlOut, xmlWrite, INDENT_3, place);
            xmlOut.println(INDENT_3 + "</efrad-person:placeOfResidence>");
        }

        xmlOut.println(INDENT_3 + "<!-- affiliations: [" + person.getAffiliations().size() + "] -->");
        for (PersonAffiliation affil : person.getAffiliations()) {
            xmlOut.print(INDENT_3 + "<efrad-person:affiliation");
            // TypeNormalVocabTime
            attribWriter.writeTypeNormalVocabTime(xmlOut, xmlWrite, INDENT_3, affil);
            xmlOut.println(INDENT_3 + "</efrad-person:affiliation>");
        }

        xmlOut.println(INDENT_3 + "<!-- addresses: [" + person.getAddresses().size() + "] -->");
        for (PersonAddress address : person.getAddresses()) {
            xmlOut.print(INDENT_3 + "<efrad-person:address");
            // TypeAvailTime
            attribWriter.writeTypeAvailTime(xmlOut, xmlWrite, INDENT_3, address);
            xmlOut.println(INDENT_3 + "</efrad-person:address>");
        }

        xmlOut.println(INDENT_3 + "<!-- languages: [" + person.getLanguages().size() + "] -->");
        for (PersonLanguage lang : person.getLanguages()) {
            xmlOut.print(INDENT_3 + "<efrad-person:languageOfPerson");
            // VocabNormalTime
            attribWriter.writeVocabNormalTime(xmlOut, xmlWrite, INDENT_3, lang);
            xmlOut.println(INDENT_3 + "</efrad-person:languageOfPerson>");
        }

        xmlOut.println(INDENT_3 + "<!-- activityFields: [" + person.getActivityFields().size() + "] -->");
        for (PersonActivityField field : person.getActivityFields()) {
            xmlOut.print(INDENT_3 + "<efrad-person:fieldOfActivity");
            // TypeNormalVocabTime
            attribWriter.writeTypeNormalVocabTime(xmlOut, xmlWrite, INDENT_3, field);
            xmlOut.println(INDENT_3 + "</efrad-person:fieldOfActivity>");
        }

        xmlOut.println(INDENT_3 + "<!-- professions: [" + person.getProfessions().size() + "] -->");
        for (PersonProfession prof : person.getProfessions()) {
            xmlOut.print(INDENT_3 + "<efrad-person:professionOccupation");
            // VocabNormalTime
            attribWriter.writeVocabNormalTime(xmlOut, xmlWrite, INDENT_3, prof);
            xmlOut.println(INDENT_3 + "</efrad-person:professionOccupation>");
        }

        xmlOut.println(INDENT_3 + "<!-- biographies: [" + person.getBiographies().size() + "] -->");
        for (PersonBiography biog : person.getBiographies()) {
            xmlOut.print(INDENT_3 + "<efrad-person:biographyHistory");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, biog);
            xmlOut.println(INDENT_3 + "</efrad-person:biographyHistory>");
        }

//            <!--
//              extensions to entity attributes
//              -->

        xmlOut.println(INDENT_3 + "<!-- notes: [" + person.getNotes().size() + "] -->");
        for (PersonNote note : person.getNotes()) {
            xmlOut.print(INDENT_3 + "<efrbr-person:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);
            xmlOut.println(INDENT_3 + "</efrbr-person:note>");
        }

        xmlOut.println(INDENT_2 + "</efrbr-person:person>");

    }
}
