/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.relation.ExpressionToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.ItemToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.ManifestationToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToResponsibleParty;
import java.io.PrintWriter;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class ResponsibleWriter {

    private static final Logger log = Logger.getLogger(ResponsibleWriter.class);

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private final AssocWriter assocWriter;

    public ResponsibleWriter() {
        this.assocWriter = new AssocWriter();
    }

    public void writeEfrbrResponsibleStart(PrintWriter xmlOut) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.println(INDENT_2 + "<efrbr:responsibleRelations>");

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.println(INDENT_2 + "<efrbr-responsible:responsibleRelations>");

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }

    public void writeEfrbrResponsibleEnd(PrintWriter xmlOut) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.println(INDENT_2 + "</efrbr:responsibleRelations>");

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.println(INDENT_2 + "</efrbr-responsible:responsibleRelations>");

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }

    public void writeEfrbrCreatedBy(PrintWriter xmlOut,
                               WorkToResponsibleParty assoc) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr:createdBy");

            assocWriter.writeSimple(
                    xmlOut, INDENT_3,
                    XMLExportContext.makeURI("work",
                                          assoc.getSourceWork().getId()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()));

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr-responsible:createdBy");

            assocWriter.writeRole(
                    xmlOut, INDENT_3,
                    "work",
                    XMLExportContext.makeURI("work",
                                          assoc.getSourceWork().getId()),
                    assocWriter.entityType(assoc.getTargetResponsibleParty()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()),
                    assoc.getRelRole());

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }

    //  <xsd:element name="realizedBy"
    //               type="GenericAssociationType"
    //               />
    public void writeEfrbrRealizedBy(PrintWriter xmlOut,
                                ExpressionToResponsibleParty assoc) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr:realizedBy");

            assocWriter.writeSimple(
                    xmlOut, INDENT_3,
                    XMLExportContext.makeURI("expression",
                                          assoc.getSourceExpression().getId()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()));

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr-responsible:realizedBy");

            assocWriter.writeRole(
                    xmlOut, INDENT_3,
                    "expression",
                    XMLExportContext.makeURI("expression",
                                          assoc.getSourceExpression().getId()),
                    assocWriter.entityType(assoc.getTargetResponsibleParty()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()),
                    assoc.getRelRole());

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }

    //  <xsd:element name="producedBy"
    //               type="GenericAssociationType"
    //               />
    public void writeEfrbrProducedBy(PrintWriter xmlOut,
                                ManifestationToResponsibleParty assoc) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr:producedBy");

            assocWriter.writeSimple(
                    xmlOut, INDENT_3,
                    XMLExportContext.makeURI("manifestation",
                                          assoc.getSourceManifestation().getId()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()));

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr-responsible:producedBy");

            assocWriter.writeRole(
                    xmlOut, INDENT_3,
                    "manifestation",
                    XMLExportContext.makeURI("manifestation",
                                          assoc.getSourceManifestation().getId()),
                    assocWriter.entityType(assoc.getTargetResponsibleParty()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()),
                    assoc.getRelRole());

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }

    //  <xsd:element name="ownedBy"
    //               type="GenericAssociationType"/>
    public void writeEfrbrOwnedBy(PrintWriter xmlOut,
                             ItemToResponsibleParty assoc) {

        if ("1.0".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr:ownedBy");

            assocWriter.writeSimple(
                    xmlOut, INDENT_3,
                    XMLExportContext.makeURI("item",
                                          assoc.getSourceItem().getId()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()));

        } else if ("1.1".equals(XMLExportContext.getVersion())) {

            xmlOut.print(INDENT_3 + "<efrbr-responsible:ownedBy");

            assocWriter.writeRole(
                    xmlOut, INDENT_3,
                    "item",
                    XMLExportContext.makeURI("item",
                                          assoc.getSourceItem().getId()),
                    assocWriter.entityType(assoc.getTargetResponsibleParty()),
                    XMLExportContext.makeURI(assocWriter.entityType(assoc.getTargetResponsibleParty()),
                                          assoc.getTargetResponsibleParty().getId()),
                    assoc.getRelRole());

        } else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }
    }
}
