/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToExpression;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToItem;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToManifestation;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToResponsibleParty;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToSubjectEntity;
import edu.indiana.dlib.vfrbr.persist.relation.WorkToWork;
import java.io.PrintWriter;

/**
 *
 * @author pbmcelwa
 */
public class SubjectWriter {

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private final AssocWriter assocWriter;

    private final XMLExportContext exportContext;

    public SubjectWriter(XMLExportContext exportContext) {
        this.assocWriter = new AssocWriter();
        this.exportContext = exportContext;
    }

    public void writeSubjectStart(PrintWriter xmlOut) {

        xmlOut.println(INDENT_2 + "<vfrbr-subject:subjectRelations>");
    }

    public void writeSubjectEnd(PrintWriter xmlOut) {

        xmlOut.println(INDENT_2 + "</vfrbr-subject:subjectRelations>");
    }

    public void writeHasSubjectWork(PrintWriter xmlOut,
                                    WorkToWork assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "work",
                exportContext.makeURI("Work",
                                   assoc.getTargetWork().getId()));
    }

    public void writeHasSubjectExpression(PrintWriter xmlOut,
                                          WorkToExpression assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "expression",
                exportContext.makeURI("Expression",
                                   assoc.getTargetExpression().getId()));
    }

    public void writeHasSubjectManifestation(PrintWriter xmlOut,
                                             WorkToManifestation assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "manifestation",
                exportContext.makeURI("Manifestation",
                                   assoc.getTargetManifestation().getId()));
    }

    public void writeHasSubjectItem(PrintWriter xmlOut,
                                    WorkToItem assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                "item",
                exportContext.makeURI("Item",
                                   assoc.getTargetItem().getId()));
    }

    public void writeHasSubjectResponsibleParty(PrintWriter xmlOut,
                                                WorkToResponsibleParty assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                assocWriter.entityType(assoc.getTargetResponsibleParty()),
                exportContext.makeURI(assoc.getTargetResponsibleParty().getClass().getSimpleName(),
                                   assoc.getTargetResponsibleParty().getId()));
    }

    public void writeHasSubjectSubjectEntity(PrintWriter xmlOut,
                                             WorkToSubjectEntity assoc) {

        xmlOut.print(INDENT_3 + "<vfrbr:hasSubject");

        assocWriter.writeSimple(
                xmlOut, INDENT_3,
                "work",
                exportContext.makeURI("Work",
                                   assoc.getSourceWork().getId()),
                assocWriter.entityType(assoc.getTargetSubjectEntity()),
                exportContext.makeURI(assoc.getTargetSubjectEntity().getClass().getSimpleName(),
                                   assoc.getTargetSubjectEntity().getId()));
    }
}
