/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import java.io.PrintWriter;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class VfrbrWriter {

    private static final Logger log = Logger.getLogger(VfrbrWriter.class);

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    public void writeDocumentStart(PrintWriter xmlOut) {

//        <?xml version="1.0" encoding="UTF-8"?>
        xmlOut.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");

    }

    public void writeRecordSetStart(PrintWriter xmlOut,
                                    XMLExportContext exportContext) {

        if ("1.1".equals(XMLExportContext.getVersion())) {

            this.writeRecSetBeg11(xmlOut);

        } else if ("1.0".equals(XMLExportContext.getVersion())) {

            this.writeRecSetBeg10(xmlOut);

        }else {
            log.warn("Unrecognized version: " + XMLExportContext.getVersion());
        }

    }

    public void writeRecordSetEnd(PrintWriter xmlOut) {

//      </vfrbr:recordSet>
        xmlOut.println("</vfrbr:recordSet>");

    }

    public void writeEntitiesStart(PrintWriter xmlOut) {

//          <vfrbr:entities>
        xmlOut.println(INDENT_1 + "<vfrbr:entities>");

    }

    public void writeEntitiesEnd(PrintWriter xmlOut) {

//          </vfrbr:entities>
        xmlOut.println(INDENT_1 + "</vfrbr:entities>");

    }

    public void writeRelationshipsStart(PrintWriter xmlOut) {

//          <vfrbr:relationships>
        xmlOut.println(INDENT_1 + "<vfrbr:relationships>");

    }

    public void writeRelationshipsEnd(PrintWriter xmlOut) {

//          </vfrbr:relationships>
        xmlOut.println(INDENT_1 + "</vfrbr:relationships>");

    }

    private void writeRecSetBeg10(PrintWriter xmlOut) {

        xmlOut.println("<vfrbr:recordSet");

        xmlOut.println(INDENT_2
                + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr=\"http://vfrbr.info/vfrbr/1.0\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-work=\"http://vfrbr.info/vfrbr/1.0/work\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-work=\"http://vfrbr.info/vfrad/1.0/work\"");

        xmlOut.println(
                INDENT_2
                + "xmlns:vfrbr-expression=\"http://vfrbr.info/vfrbr/1.0/expression\"");

        xmlOut.println(
                INDENT_2
                + "xmlns:vfrbr-manifestation=\"http://vfrbr.info/vfrbr/1.0/manifestation\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-item=\"http://vfrbr.info/vfrbr/1.0/item\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-item=\"http://vfrbr.info/vfrad/1.0/item\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-person=\"http://vfrbr.info/vfrbr/1.0/person\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-person=\"http://vfrbr.info/vfrad/1.0/person\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-family=\"http://vfrbr.info/vfrbr/1.0/family\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-family=\"http://vfrbr.info/vfrad/1.0/family\"");

        xmlOut.println(
                INDENT_2
                + "xmlns:vfrbr-corporateBody=\"http://vfrbr.info/vfrbr/1.0/corporateBody\"");

        xmlOut.println(
                INDENT_2
                + "xmlns:vfrad-corporateBody=\"http://vfrbr.info/vfrad/1.0/corporateBody\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-concept=\"http://vfrbr.info/vfrbr/1.0/concept\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-object=\"http://vfrbr.info/vfrbr/1.0/object\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-event=\"http://vfrbr.info/vfrbr/1.0/event\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-place=\"http://vfrbr.info/vfrbr/1.0/place\"");

        xmlOut.println("\n" + INDENT_2 + "xsi:schemaLocation=");

        xmlOut.println(
                INDENT_3
                + "\"http://vfrbr.info/vfrbr/1.0 http://vfrbr.info/schemas/1.0/vfrbr.xsd\"");

        /*
         * other schema locations come with vfrbr.xsd
         */
        xmlOut.println(INDENT_2 + ">");
    }

    private void writeRecSetBeg11(PrintWriter xmlOut) {

        xmlOut.println("<vfrbr:recordSet");

        xmlOut.println(INDENT_2
                + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr=\"http://vfrbr.info/vfrbr/1.1\"");

        // -- entities --

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-work=\"http://vfrbr.info/vfrbr/1.1/work\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-work=\"http://vfrbr.info/vfrad/1.1/work\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-expression=\"http://vfrbr.info/vfrbr/1.1/expression\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-manifestation=\"http://vfrbr.info/vfrbr/1.1/manifestation\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-item=\"http://vfrbr.info/vfrbr/1.1/item\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-item=\"http://vfrbr.info/vfrad/1.1/item\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-person=\"http://vfrbr.info/vfrbr/1.1/person\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-person=\"http://vfrbr.info/vfrad/1.1/person\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-family=\"http://vfrbr.info/vfrbr/1.1/family\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-family=\"http://vfrbr.info/vfrad/1.1/family\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-corporateBody=\"http://vfrbr.info/vfrbr/1.1/corporateBody\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-corporateBody=\"http://vfrbr.info/vfrad/1.1/corporateBody\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-concept=\"http://vfrbr.info/vfrbr/1.1/concept\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-object=\"http://vfrbr.info/vfrbr/1.1/object\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-event=\"http://vfrbr.info/vfrbr/1.1/event\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-place=\"http://vfrbr.info/vfrbr/1.1/place\"");

        // -- relations --

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-structure=\"http://vfrbr.info/vfrbr/1.1/structure\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-responsible=\"http://vfrbr.info/vfrbr/1.1/responsible\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-subject=\"http://vfrbr.info/vfrbr/1.1/subject\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrbr-other=\"http://vfrbr.info/vfrbr/1.1/other\"");

        xmlOut.println(INDENT_2
                + "xmlns:vfrad-other=\"http://vfrbr.info/vfrad/1.1/other\"");

        // -- schema location --

        xmlOut.println("\n" + INDENT_2 + "xsi:schemaLocation=");

        xmlOut.println(
                INDENT_3
                + "\"http://vfrbr.info/vfrbr/1.1 http://vfrbr.info/schemas/1.1/vfrbr.xsd\"");

        xmlOut.println(INDENT_2 + ">");
    }
}
