/**
 * Copyright 2009-2011, Trustees of Indiana University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *   Neither the name of Indiana University nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package edu.indiana.dlib.vfrbr.export.xml.writer;

import edu.indiana.dlib.vfrbr.export.context.XMLExportContext;
import edu.indiana.dlib.vfrbr.persist.entity.work.*;

import java.io.PrintWriter;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;
import org.apache.log4j.Logger;

/**
 *
 * @author pbmcelwa
 */
public class WorkWriter {

    private static final Logger log = Logger.getLogger(WorkWriter.class);

    private static String INDENT_1 = "  ";

    private static String INDENT_2 = "    ";

    private static String INDENT_3 = "      ";

    private static String INDENT_4 = "        ";

    private static String INDENT_5 = "          ";

    private final AttribWriter attribWriter;
    
    public WorkWriter() {
        this.attribWriter = new AttribWriter();
    }

    public void writeEfrbrWork(PrintWriter xmlOut,
                          XMLStreamWriter xmlWrite,
                          WorkJpa work)
            throws XMLStreamException {

        xmlOut.print(INDENT_2 + "<efrbr-work:work");

        xmlOut.print("\n" + INDENT_2 + INDENT_2
                + "identifier=\""
                + XMLExportContext.makeURI("work", work.getId())
                + "\"");

        xmlOut.println(">");

        xmlOut.println(INDENT_3 + "<!-- titles: ["
                + work.getTitles().size() + "] -->");
        for (WorkTitle title : work.getTitles()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:titleOfTheWork");
            // TypeVocabOffset
            attribWriter.writeTypeVocabOffset(xmlOut, xmlWrite, INDENT_3, title);
            xmlOut.println(INDENT_3 + "</efrbr-work:titleOfTheWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- forms: ["
                + work.getForms().size() + "] -->");
        for (WorkForm form : work.getForms()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:formOfWork");
            // Vocab
            attribWriter.writeVocab(xmlOut, xmlWrite, INDENT_3, form);
            xmlOut.println(INDENT_3 + "</efrbr-work:formOfWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- dates: ["
                + work.getDates().size() + "] -->");
        for (WorkDate date : work.getDates()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:dateOfTheWork");
            // TypeNormal
            attribWriter.writeTypeNormal(xmlOut, xmlWrite, INDENT_3, date);

            xmlOut.println(INDENT_3 + "</efrbr-work:dateOfTheWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- characteristics: ["
                + work.getCharacteristics().size() + "] -->");
        for (WorkCharacteristic charac : work.getCharacteristics()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:otherDistinguishingCharacteristic");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, charac);
            xmlOut.println(INDENT_3 + "</efrbr-work:otherDistinguishingCharacteristic>");
        }

//        <!-- excluded
//        <xsd:element
//          ref="intendedTermination"
//          minOccurs="0" maxOccurs="unbounded"/>
//          -->


        xmlOut.println(INDENT_3 + "<!-- audiences: ["
                + work.getAudiences().size() + "] -->");
        for (WorkAudience audience : work.getAudiences()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:intendedAudience");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, audience);
            xmlOut.println(INDENT_3 + "</efrbr-work:intendedAudience>");
        }

        xmlOut.println(INDENT_3 + "<!-- contexts: ["
                + work.getContexts().size() + "] -->");
        for (WorkContext context : work.getContexts()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:contextForTheWork");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, context);
            xmlOut.println(INDENT_3 + "</efrbr-work:contextForTheWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- performanceMediums: ["
                + work.getPerformanceMediums().size() + "] -->");
        for (WorkPerformanceMedium perfMed : work.getPerformanceMediums()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:mediumOfPerformance");
            // VocabQuant
            attribWriter.writeVocabQuant(xmlOut, xmlWrite, INDENT_3, perfMed);
            xmlOut.println(INDENT_3 + "</efrbr-work:mediumOfPerformance>");
        }

        xmlOut.println(INDENT_3 + "<!-- designations: ["
                + work.getDesignations().size() + "] -->");
        for (WorkDesignation desig : work.getDesignations()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:numericDesignation");
            // Type
            attribWriter.writeType(xmlOut, xmlWrite, INDENT_3, desig);
            xmlOut.println(INDENT_3 + "</efrbr-work:numericDesignation>");
        }

        xmlOut.println(INDENT_3 + "<!-- keys: ["
                + work.getKeys().size() + "] -->");
        for (WorkKey key : work.getKeys()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:key");
            // Vocab
            attribWriter.writeVocab(xmlOut, xmlWrite, INDENT_3, key);
            xmlOut.println(INDENT_3 + "</efrbr-work:key>");
        }

//        <!-- excluded
//        <xsd:element
//          ref="coordinates"
//          minOccurs="0" maxOccurs="unbounded"/>
//        <xsd:element
//          ref="equinox"
//          minOccurs="0" maxOccurs="unbounded"/>
//          -->

//        <!-- FRAD attributes -->

        xmlOut.println(INDENT_3 + "<!-- subjectHeadings: ["
                + work.getSubjectHeadings().size() + "] -->");
        for (WorkSubjectHeading subjHead : work.getSubjectHeadings()) {
            xmlOut.print(INDENT_3 + "<efrad-work:subjectOfTheWork");
            // TypeVocab
            attribWriter.writeTypeVocab(xmlOut, xmlWrite, INDENT_3, subjHead);
            xmlOut.println(INDENT_3 + "</efrad-work:subjectOfTheWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- originPlaces: ["
                + work.getOriginPlaces().size() + "] -->");
        for (WorkOriginPlace origPlace : work.getOriginPlaces()) {
            xmlOut.print(INDENT_3 + "<efrad-work:placeOfOriginOfTheWork");
            // JurisNormalVocab
            attribWriter.writeJurisNormalVocab(xmlOut, xmlWrite, INDENT_3, origPlace);
            xmlOut.println(INDENT_3 + "</efrad-work:placeOfOriginOfTheWork>");
        }

        xmlOut.println(INDENT_3 + "<!-- histories: ["
                + work.getHistories().size() + "] -->");
        for (WorkHistory history : work.getHistories()) {
            xmlOut.print(INDENT_3 + "<efrad-work:history");
            // Simple
            attribWriter.writeSimple(xmlOut, xmlWrite, INDENT_3, history);
            xmlOut.println(INDENT_3 + "</efrad-work:history>");
        }

//        <!--
//          extensions to entity attributes
//          -->

        xmlOut.println(INDENT_3 + "<!-- notes: ["
                + work.getNotes().size() + "] -->");
        for (WorkNote note : work.getNotes()) {
            xmlOut.print(INDENT_3 + "<efrbr-work:note");
            // TypeAvail
            attribWriter.writeTypeAvail(xmlOut, xmlWrite, INDENT_3, note);
            xmlOut.println(INDENT_3 + "</efrbr-work:note>");
        }

        xmlOut.println(INDENT_2 + "</efrbr-work:work>");

    }
}
